package org.rcisoft.integration.jieLink.service.impl;


import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.integration.jieLink.common.SignUtils;
import org.rcisoft.integration.jieLink.dto.*;
import org.rcisoft.integration.jieLink.service.IJieLinkService;
import org.rcisoft.sys.devices.entity.Devices;
import org.rcisoft.sys.devices.service.DevicesService;
import org.rcisoft.sys.sysuser.dao.SysDictDataRepositorys;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.entity.DoorRecord;
import org.rcisoft.sys.sysuser.entity.SysDictData;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.DoorRecordService;
import org.rcisoft.sys.sysuser.service.SysUserService;
import org.rcisoft.sys.sysusercard.entity.SysUserCard;
import org.rcisoft.sys.sysusercard.service.SysUserCardService;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;

/**
 * 捷顺道闸对接接口实现
 */
@Service
@Slf4j
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class JieLinkServiceImpl implements IJieLinkService {

    /**
     * 捷顺服务地址
     */
    @Value("${jieLink.apiUrl}")
    private String jieLinkUri;

    /**
     * 接口用户
     */
    @Value("${jieLink.account}")
    private String jieLinkUser;

    /**
     * 接口密码
     */
    @Value("${jieLink.pwd}")
    private String jieLinkPwd;


    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private DoorRecordService doorRecordService;

    @Autowired
    private DevicesService devicesService;

    @Autowired
    private SysDictDataRepositorys dictDataRepositorys;

    @Autowired
    private SysUserCardService sysUserCardService;

    @Autowired
    private SysUserRepositorys sysUserRepositorys;

    @Value("${global.path.base_discovery}")
    private String baseDiscovery;
    @Value("${global.path.base_upload_location}")
    private String baseUploadLocation;

    /**
     * 获取签名所需相关参数
     *
     * @return
     */
    @Override
    public SignDto getAppKeyInfo() {
        String apiUri = "/api/internal/sign";
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
        headers.set("Accept", MediaType.APPLICATION_JSON_VALUE);
        JSONObject paramMap = new JSONObject();
        paramMap.put("userName", jieLinkUser);
        paramMap.put("password", jieLinkPwd);
        HttpEntity<JSONObject> httpEntity = new HttpEntity<>(paramMap, headers);
        StringBuffer uriAddr = new StringBuffer();
        uriAddr.append(jieLinkUri).append(apiUri);
        try {
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            if (HttpStatus.OK.value() == responseEntity.getStatusCode().value()) {
                log.debug("getAppKeyInfo接口返回信息：" + responseEntity.getBody().toJSONString());
                JSONObject body = responseEntity.getBody();
                if (StringUtils.equals(body.getString("code"), "0")) {
                    List<AppInfoDto> appInfoDtoList = JSONArray.parseArray(body.getJSONArray("data").toJSONString(), AppInfoDto.class);
                    if (appInfoDtoList != null && !appInfoDtoList.isEmpty()) {
                        AppInfoDto appInfoDto = appInfoDtoList.get(0);
                        SignDto signData = new SignDto();
                        signData.setAppId(appInfoDto.getAppId());
                        signData.setKey(appInfoDto.getKey());
                        return signData;
                    } else {
                        //返回结果为空
                        return null;
                    }

                } else {
                    log.error("getAppKeyInfo接口返回错误信息：" + body.getString("msg"));
                    //请求返回结果错误啦
                    return null;
                }
            } else {
                //请求失败啦
                log.error("getAppKeyInfo接口请求失败");
                return null;
            }
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            return null;
        }
    }

    /**
     * 推送新增用户对象到捷顺服务中
     *
     * @param userData
     * @return
     */
    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    public boolean pushAddUserData(SysUser userData) {
        String apiUri = "/api/base/addperson";
        HttpHeaders headers = this.loadHeaderHandle();
        if (headers != null) {
            JieLinkPersonDto pushData = this.convertUserBean(userData);
            HttpEntity<JieLinkPersonDto> httpEntity = new HttpEntity<>(pushData, headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    log.debug("pushAddUserData接口返回信息："+responseEntity.getBody().toJSONString());
                    JSONObject body = responseEntity.getBody();
                    if(body.containsKey("code") && StringUtils.equals("0",body.getString("code"))){
                        JSONObject data = body.getJSONObject("data");
                        JieLinkPersonDto persion = JSONObject.parseObject(data.toJSONString(),JieLinkPersonDto.class);
                        if(persion!=null){
                            //此处需要将捷顺中的ID存入到用户信息中
                            String personId = persion.getPersonId();
                            if(userData.getBusinessId()!=null){
                                sysUserService.saveJieLinkId(personId,userData.getBusinessId());
                                //下面开始开通授权
                                Integer businessId = userData.getBusinessId();
                                boolean openCertStatus = this.openCertificateServiceData(personId, businessId.longValue());
                                if(openCertStatus){
                                    return true;
                                }else{
                                    return false;
                                }
                            }else{
                                //ID竟然没有返回，说明参数没有传全
                                log.error("pushAddUserData接口请求时缺少用户ID");
                                return false;
                            }
                        } else {
                            //返回结果为空
                            return false;
                        }
                    }else{
                        log.error("pushAddUserData接口返回错误信息："+body.getString("msg"));
                        //请求返回结果错误啦
                        return false;
                    }
                } else {
                    //请求失败啦
                    log.error("pushAddUserData接口请求失败");
                    return false;
                }
            } catch (Exception ex) {
                log.error(ex.getMessage(), ex);
                return false;
            }
        } else {
            //认证信息获取失败
            log.error("pushAddUserData接口获取认证参数失败");
            return false;
        }
    }


    /**
     * 推送更新的用户信息到接口服务中
     *
     * @param userData
     * @return
     */
    @Override
    public boolean pushModifyUserData(SysUser userData) {
        if (userData != null && StringUtils.isNotBlank(userData.getPersonId())) {
            String apiUri = "/api/base/updateperson";
            HttpHeaders headers = this.loadHeaderHandle();
            if (headers != null) {
                JieLinkPersonDto pushData = this.convertUserBean(userData);
                pushData.setPersonId(userData.getPersonId());
                //此处写死部门
                pushData.setDeptId("5a9dc528-e7ad-4952-9f21-a885b6564a1c");
                HttpEntity<JieLinkPersonDto> httpEntity = new HttpEntity<>(pushData, headers);
                StringBuffer uriAddr = new StringBuffer();
                uriAddr.append(jieLinkUri).append(apiUri);
                ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
                try {
                    if (HttpStatus.OK.value() == responseEntity.getStatusCode().value()) {
                        log.debug("pushModifyUserData接口返回信息：" + responseEntity.getBody().toJSONString());
                        JieLinkResultDto<JieLinkPersonDto> resultData = JSONObject.toJavaObject(responseEntity.getBody(), JieLinkResultDto.class);
                        if (StringUtils.equals(resultData.getCode(), "0")) {
                            return true;
                        } else {
                            log.error("pushAddUserData接口返回错误信息：" + resultData.getMsg());
                            //请求返回结果错误啦
                            return false;
                        }
                    } else {
                        //请求失败啦
                        log.error("pushAddUserData接口请求失败");
                        return false;
                    }
                } catch (Exception ex) {
                    log.error(ex.getMessage(), ex);
                    return false;
                }
            } else {
                //认证信息获取失败
                log.error("pushAddUserData接口获取认证参数失败");
                return false;
            }
        } else {
            log.error("当前用户没有捷顺系统的ID，无法更新");
            return true;
        }
    }

    @Override
    public boolean pushDelUserData(SysUser userData) {
        if (userData != null && StringUtils.isNotBlank(userData.getPersonId())) {
            //1、查询用户核心数据
            SysUser userInfo = sysUserService.getUserById(userData.getBusinessId());
            String apiUri = "/api/base/deleteperson";
            HttpHeaders headers = this.loadHeaderHandle();
            if (headers != null) {
                JSONObject paramMap = new JSONObject();
                paramMap.put("personId", userData.getPersonId());
                HttpEntity<JSONObject> httpEntity = new HttpEntity<>(paramMap, headers);
                StringBuffer uriAddr = new StringBuffer();
                uriAddr.append(jieLinkUri).append(apiUri);
                ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
                try {
                    if (HttpStatus.OK.value() == responseEntity.getStatusCode().value()) {
                        log.debug("pushDelUserData接口返回信息：" + responseEntity.getBody().toJSONString());
                        JieLinkResultDto<JieLinkPersonDto> resultData = JSONObject.toJavaObject(responseEntity.getBody(), JieLinkResultDto.class);
                        if (StringUtils.equals(resultData.getCode(), "0")) {
                            //删除用户时，删除授权凭证
                            //TODO 临时去掉该逻辑，后续等上线时放开
//                            boolean closeCertStatus = this.closeCertificateServiceData(userInfo);
                            boolean closeCertStatus = true;
                            if(closeCertStatus){
                                return true;
                            }else{
                                return false;
                            }
                        } else {
                            log.error("pushDelUserData接口返回错误信息：" + resultData.getMsg());
                            //请求返回结果错误啦
                            return false;
                        }
                    } else {
                        //请求失败啦
                        log.error("pushDelUserData接口请求失败");
                        return false;
                    }
                } catch (Exception ex) {
                    log.error(ex.getMessage(), ex);
                    return false;
                }
            } else {
                //认证信息获取失败
                log.error("pushDelUserData接口获取认证参数失败");
                return false;
            }
        } else {
            log.error("当前用户没有捷顺系统的ID，无法更新");
            return true;
        }
    }


    /**
     * 开通门禁服务
     * @param personId
     * @return
     */
    @Override
    public boolean openCertificateServiceData(String personId,Long userId) {
        String apiUri = "/api/door/downdoorcredential";
        HttpHeaders headers = this.loadHeaderHandle();
        if (headers != null) {
            SysUserCard cardData = new SysUserCard();
            cardData.setPersonId(personId);
            cardData.setUserId(userId);
            JSONArray certificateDataList = createCertificateData(cardData);
            JSONObject paramMap = new JSONObject();
            paramMap.put("cardServices",certificateDataList);
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(paramMap, headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try {
                if (HttpStatus.OK.value() == responseEntity.getStatusCode().value()) {
                    log.info("openCertificateServiceData接口返回信息：" + responseEntity.getBody().toJSONString());
                    JSONObject body = responseEntity.getBody();
                    if(body.containsKey("code")&& StringUtils.equals("0",body.getString("code"))){
                        JSONObject data = body.getJSONObject("data");
                        if(data.containsKey("resultList")){
                            JSONArray resultList = data.getJSONArray("resultList");
                            boolean allStatus = true;
                            for(int dataIndex = 0; dataIndex<resultList.size();dataIndex++){
                                JSONObject jsonObject = resultList.getJSONObject(dataIndex);
                                if(jsonObject.containsKey("result") && jsonObject.getIntValue("jsonObject")==0){
                                    allStatus = allStatus && true;
                                }else{
                                    log.error("openCertificateServiceData接口操作失败："+jsonObject.getString("message"));
                                    allStatus = allStatus && false;
                                }
                            }
                            if(allStatus){
                                sysUserCardService.persist(cardData);//保存凭证信息
                            }
                            //此处返回总的结果
                            return allStatus;
                        }else{
                            log.error("openCertificateServiceData接口返回数据为空：");
                            //请求返回结果错误啦
                            return false;
                        }
                    }else {
                        log.error("openCertificateServiceData接口返回错误信息：" + body.getString("msg"));
                        //请求返回结果错误啦
                        return false;
                    }

                } else {
                    //请求失败啦
                    log.error("openCertificateServiceData接口请求失败");
                    return false;
                }
            } catch (Exception ex) {
                log.error(ex.getMessage(), ex);
                return false;
            }
        } else {
            //认证信息获取失败
            log.error("openCertificateServiceData接口获取认证参数失败");
            return false;
        }
    }

    /**
     * 生成用户的凭证信息
     * @param cardData
     * @return
     */
    private JSONArray createCertificateData(SysUserCard cardData){
        JSONArray dataList = new JSONArray();
        //组装数据
        JSONObject cardService = new JSONObject();
        cardData.setCertificateId(UUID.randomUUID().toString());
        cardService.put("itemId",cardData.getCertificateId());//凭证ID
        //设备
        JSONArray devicesArray = new JSONArray();
        deviceGuidAllArray.forEach(deviceMap->{
            JSONObject device = new JSONObject();
            device.put("deviceGuid",deviceMap.get("deviceApGuid"));
            device.put("isDeleted",0);
            devicesArray.add(device);
        });
        cardService.put("deviceList",devicesArray);
        //开通状态
        cardService.put("isDeleted",0);
        //凭证类型
        cardService.put("credentialType",1);
        cardData.setCertificateType("1");//实体卡
        cardData.setCertificateNo(RandomUtil.randomString("ABCDEF0123456789", 8));
        //凭证卡号
        cardService.put("credentialNo",cardData.getCertificateNo());
        //凭证状态 开通
        cardService.put("credentialStatus","NORMAL");
        //人员ID
        cardService.put("personId",cardData.getPersonId());
        //有效日期
        Date now = new Date();
        DateTime startTime = DateUtil.beginOfYear(now);
        cardService.put("serviceStartDate",DateUtil.format(startTime,"yyyy-MM-dd"));
        //结束日期
        DateTime endTime = DateUtil.offset(startTime, DateField.DAY_OF_YEAR, 8);
        cardData.setCertificateEndTime(endTime);
        cardService.put("serviceEndDate",DateUtil.format(endTime,"yyyy-MM-dd"));
        //有效星期
        cardService.put("week","1111111");
        cardData.setCertificatePwd(RandomUtil.randomString("123456789",4));
        cardService.put("cardKey",cardData.getCertificatePwd());
        dataList.add(cardService);
        return dataList;
    }


    /**
     * 关闭门禁服务
     * @param userData
     * @return
     */
    @Override
    public boolean closeCertificateServiceData(SysUser userData) {
        /**
         * 1、通过用户名称获取所有凭证信息
         * 2、关闭所有凭证信息
         */
        JSONArray certificateData = this.loadUserCertificateData(userData);
        if(certificateData!=null){
            if(!certificateData.isEmpty()){
                JSONArray delCertData = createUserDelCertData(userData, certificateData);
                String apiUri = "/api/door/downdoorcredential";
                HttpHeaders headers = this.loadHeaderHandle();
                if (headers != null) {
                    JSONObject paramMap = new JSONObject();
                    paramMap.put("cardServices",delCertData);
                    HttpEntity<JSONObject> httpEntity = new HttpEntity<>(paramMap, headers);
                    StringBuffer uriAddr = new StringBuffer();
                    uriAddr.append(jieLinkUri).append(apiUri);
                    ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
                    try {
                        if (HttpStatus.OK.value() == responseEntity.getStatusCode().value()) {
                            log.info("closeCertificateServiceData接口返回信息：" + responseEntity.getBody().toJSONString());
                            JSONObject body = responseEntity.getBody();
                            if(body.containsKey("code")&& StringUtils.equals("0",body.getString("code"))){
                                JSONObject data = body.getJSONObject("data");
                                if(data.containsKey("resultList")){
                                    JSONArray resultList = data.getJSONArray("resultList");
                                    boolean allStatus = true;
                                    for(int dataIndex = 0; dataIndex<resultList.size();dataIndex++){
                                        JSONObject jsonObject = resultList.getJSONObject(dataIndex);
                                        if(jsonObject.containsKey("result") && jsonObject.getIntValue("jsonObject")==0){
                                            allStatus = allStatus && true;
                                        }else{
                                            log.error("closeCertificateServiceData接口操作失败："+jsonObject.getString("message"));
                                            allStatus = allStatus && false;
                                        }
                                    }
                                    //此处返回总的结果
                                    if(allStatus){
                                        sysUserCardService.delUserCardByUserId(null,userData.getPersonId());
                                    }
                                    return allStatus;
                                }else{
                                    log.error("closeCertificateServiceData接口返回数据为空：");
                                    //请求返回结果错误啦
                                    return false;
                                }
                            }else {
                                log.error("closeCertificateServiceData接口返回错误信息：" + body.getString("msg"));
                                //请求返回结果错误啦
                                return false;
                            }
                        } else {
                            //请求失败啦
                            log.error("closeCertificateServiceData接口请求失败");
                            return false;
                        }
                    } catch (Exception ex) {
                        log.error(ex.getMessage(), ex);
                        return false;
                    }
                } else {
                    //认证信息获取失败
                    log.error("closeCertificateServiceData接口获取认证参数失败");
                    return false;
                }
            }else{
                //没有凭证可删除
                log.info("当前用户无凭证可删除");
                return true;
            }
        }else{
            return false;
        }
    }

    /**
     * 生成
     * @param userData
     * @param certArray
     * @return
     */
    private JSONArray createUserDelCertData(SysUser userData,JSONArray certArray){
        JSONArray pushArrayData = new JSONArray();
        for(int certIndex = 0 ; certIndex < certArray.size(); certIndex++){
            JSONObject certObj = certArray.getJSONObject(certIndex);
            JSONObject cardService = new JSONObject();
            cardService.put("itemId",UUID.randomUUID().toString());//凭证ID
            //设备
            JSONArray devicesArray = new JSONArray();
            deviceGuidAllArray.forEach(deviceMap->{
                JSONObject device = new JSONObject();
                device.put("deviceGuid",deviceMap.get("deviceApGuid"));
                device.put("isDeleted",1);
                devicesArray.add(device);
            });
            cardService.put("deviceList",devicesArray);
            //开通状态
            cardService.put("isDeleted",0);
            //凭证类型
            cardService.put("credentialType",certObj.getIntValue("voucherType"));
            //凭证卡号
            cardService.put("credentialNo",certObj.getString("voucherNo"));
            //凭证状态 开通
            cardService.put("credentialStatus","BACKED");
            //人员ID
            cardService.put("personId",userData.getPersonId());
            //有效日期
            Date now = new Date();
            DateTime startTime = DateUtil.beginOfYear(now);
            cardService.put("serviceStartDate",DateUtil.format(startTime,"yyyy-MM-dd"));
            //结束日期
            DateTime endTime = DateUtil.offset(startTime, DateField.DAY_OF_YEAR, 8);
            cardService.put("serviceEndDate",DateUtil.format(endTime,"yyyy-MM-dd"));
            //有效星期
            cardService.put("week","1111111");
            cardService.put("cardKey",RandomUtil.randomString("123456789",4));
            pushArrayData.add(cardService);
        }
        return pushArrayData;
    }


    @Override
    public JSONArray loadUserCertificateData(SysUser userData) {
        String apiUri = "/api/base/vouchers";
        HttpHeaders headers = this.loadHeaderHandle();
        if (headers != null) {
            JSONObject pushData = new JSONObject();
            pushData.put("pageIndex",1);
            pushData.put("pageSize",50);
            pushData.put("personName",userData.getName());//根据名字查询
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData,headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    log.info("loadUserCertificateData接口返回信息："+responseEntity.getBody().toJSONString());
                    JSONObject body = responseEntity.getBody();
                    if(body.containsKey("code") && StringUtils.equals("0",body.getString("code"))){
                        JSONObject data = body.getJSONObject("data");
                        if(data.containsKey("voucherList")){
                            JSONArray voucherList = data.getJSONArray("voucherList");
                            JSONArray resultArray = new JSONArray();
                            for(int dataIndex =0; dataIndex<voucherList.size();dataIndex++){
                                JSONObject certificateData = voucherList.getJSONObject(dataIndex);
                                if(StringUtils.equals(userData.getUsername(),certificateData.getString("personNo"))){
                                    resultArray.add(certificateData);
                                }
                            }
                            return resultArray;
                        }else{
                            log.error("loadUserCertificateData接口请求数据返回为空");
                            return new JSONArray();
                        }
                    }else{
                        log.error("loadUserCertificateData接口请求数据返回错误："+body.getString("msg"));
                        return null;
                    }
                }else{
                    log.error("loadUserCertificateData接口请求失败");
                    return null;
                }
            }catch (Exception ex){
                log.error(ex.getMessage(),ex);
                return null;
            }
        } else {
            //认证信息获取失败
            log.error("loadUserCertificateData接口获取认证参数失败");
            return null;
        }
    }

    /**
     * 用户对象转换
     *
     * @param userData
     * @return
     */
    private JieLinkPersonDto convertUserBean(SysUser userData) {
        JieLinkPersonDto pushData = new JieLinkPersonDto();
        pushData.setPersonNo(userData.getUsername());//人员编号
        pushData.setPersonName(userData.getName());//姓名 必填
        pushData.setPersonGender(StringUtils.equals("0", userData.getSex()) ? 0 : StringUtils.equals("1", userData.getSex()) ? 1 : null);//性别
        //此处需要将头像信息转换成BASE64数据存储
        if (StringUtils.isNotBlank(userData.getFaceAddress())) {
            //读取文件将文件转换为base64字符串
            String path = userData.getFaceAddress().replaceAll(baseDiscovery, baseUploadLocation);
            log.error("path-----------" + path);
            String picBase64Str = encryptToBase64(path);
            if (StringUtils.isNotBlank(picBase64Str)) {
                pushData.setPersonPhoto(picBase64Str);
            }
            log.error("图片转换成功");
        }
        //身份证处理
        if (StringUtils.isNotBlank(userData.getIdNumber())) {
            if (userData.getIdNumber().trim().length() == 15) {
                pushData.setCertificateType("GENERIDENT");//一代身份证
            } else {
                //其他类型不符合长度要求，默认按照二代身份证处理
                pushData.setCertificateType("IDENTITY");//默认二代身份证
            }
            pushData.setIdentityNo(userData.getIdNumber());//身份证
        }
        pushData.setMobile(userData.getPhone());//必填
        pushData.setEmail(userData.getEmail());
        if (StringUtils.isNotBlank(userData.getUserType())) {
            if (StringUtils.equals("0", userData.getUserType())) {
                pushData.setTenementType(1);//正式员工
            } else {
                pushData.setTenementType(5);//非正式员工以外全部定义为临时人员
            }
        }
        return pushData;
    }


    /**
     * 推送方可邀请记录到接口服务中
     *
     * @param visitInfoMation
     * @return
     */
    @Override
    public boolean pushVisitorData(VisitInfoMation visitInfoMation) {
        String apiUri = "/api/base/visitordatainvite";
        HttpHeaders headers = this.loadHeaderHandle();
        if (headers != null) {
            JSONObject pushData = this.convertVisitorData(visitInfoMation);
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData, headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try {
                if (HttpStatus.OK.value() == responseEntity.getStatusCode().value()) {
                    //后续处理逻辑
                    log.debug("pushVisitorData接口返回信息：" + responseEntity.getBody().toJSONString());
                    JieLinkResultDto<JSONObject> resultData = JSONObject.toJavaObject(responseEntity.getBody(), JieLinkResultDto.class);
                    if (StringUtils.equals(resultData.getCode(), "0")) {
                        return true;
                    } else {
                        log.error("pushVisitorData接口返回错误信息：" + resultData.getMsg());
                        //请求返回结果错误啦
                        return false;
                    }
                } else {
                    //请求失败啦
                    log.error("pushVisitorData接口请求失败");
                    return false;
                }
            } catch (Exception ex) {
                log.error(ex.getMessage(), ex);
                return false;
            }
        } else {
            //认证信息获取失败
            log.error("pushVisitorData接口获取认证参数失败");
            return false;
        }
    }


    /**
     * 撤销邀访记录
     *
     * @param visitUserId 邀访人Id
     * @param visitInfoId 邀请记录ID
     * @return
     */
    @Override
    public boolean cancelVisitorData(String visitUserId, String visitInfoId) {
        String apiUri = "/api/base/visitdatacancelinvite";
        HttpHeaders headers = this.loadHeaderHandle();
        if (headers != null) {
            JSONObject pushData = new JSONObject();
            if (StringUtils.isNotBlank(visitInfoId)) {
                pushData.put("inviteId", visitInfoId);
            } else if (StringUtils.isNotBlank(visitUserId)) {
                pushData.put("visitorId", visitUserId);
            } else {
                //
                log.error("cancelVisitorData 接口传递的参数信息有误，请检查参数");
                return false;
            }
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData, headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try {
                if (HttpStatus.OK.value() == responseEntity.getStatusCode().value()) {
                    log.debug("cancelVisitorData接口返回信息：" + responseEntity.getBody().toJSONString());
                    JieLinkResultDto resultData = JSONObject.toJavaObject(responseEntity.getBody(), JieLinkResultDto.class);
                    if (StringUtils.equals(resultData.getCode(), "0")) {
                        return true;
                    } else {
                        log.error("cancelVisitorData接口返回错误信息：" + resultData.getMsg());
                        //请求返回结果错误啦
                        return false;
                    }
                } else {
                    //请求失败啦
                    log.error("cancelVisitorData接口请求失败");
                    return false;
                }
            } catch (Exception ex) {
                log.error(ex.getMessage(), ex);
                return false;
            }
        } else {
            //认证信息获取失败
            log.error("cancelVisitorData接口获取认证参数失败");
            return false;
        }
    }

    /**
     * 所有设备ID
     */
    public static List<Map<String, String>> deviceGuidAllArray = new ArrayList<>();


    /**
     * 门口设备集合
     */
    public static List<Map<String, String>> deviceGuidInArray = new ArrayList<>();

    static {
        deviceGuidAllArray.add(new HashMap() {{
            put("deviceApGuid", "59c50c9c-3d75-420f-be9b-ad2db1aee432");
        }});//右出口
        deviceGuidAllArray.add(new HashMap() {{
            put("deviceApGuid", "64467dcf-9e59-495e-b9e4-f65a98732a99");
        }});//右入口
        deviceGuidAllArray.add(new HashMap() {{
            put("deviceApGuid", "6e2bfaad-feb3-4171-9b85-86f00589bcce");
        }});//左入口
        deviceGuidAllArray.add(new HashMap() {{
            put("deviceApGuid", "e90c55ec-bc68-498d-966c-66091ee85c27");
        }});//左出口
        deviceGuidAllArray.add(new HashMap() {{
            put("deviceApGuid", "754fdf10-be59-4004-b869-969dbc61987d");
        }});//大门出口
        deviceGuidAllArray.add(new HashMap() {{
            put("deviceApGuid", "a3dc7973-a64c-4359-bdc9-a19fe32a9f9d");
        }});//大门入口
        //门口访客机器
        deviceGuidInArray.add(new HashMap() {{
            put("deviceApGuid", "");
        }});

    }


    /**
     * 邀访记录转换
     *
     * @param infoMation
     * @return
     */
    private JSONObject convertVisitorData(VisitInfoMation infoMation) {
        JSONObject result = new JSONObject();
        VisitDataInviteDto manData = new VisitDataInviteDto();
        manData.setPersonId(sysUserService.getPersionIdByUserId(infoMation.getUserId()));//转换ID
        manData.setItemId(infoMation.getItemId());//邀访记录
        //TODO 生成邀访时间格式的JSON字符串
        Date visitRealTime = infoMation.getVisitRealTime();
        SysDictData queryDicData = new SysDictData();
        queryDicData.setDictType("timeThreshold");
        queryDicData.setDelFlag("0");
        queryDicData.setFlag("1");
        List<SysDictData> dictDataList = dictDataRepositorys.querySysDictDatas(queryDicData);
        JSONObject timeObj = new JSONObject();
        for (SysDictData dicData : dictDataList) {
            String dictValue = dicData.getDictValue();
            //此处可能会有类型转换报错的BUG
            int factor = Integer.parseInt(dictValue);
            String dictLabel = dicData.getDictLabel();
            if (StringUtils.equals("advance", dictLabel)) {
                Date startTime = new Date(visitRealTime.getTime() - (factor * 60 * 1000));
                timeObj.put("sd", DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"));
            } else if (StringUtils.equals("late", dictLabel)) {
                Date startTime = new Date(visitRealTime.getTime() + (factor * 60 * 1000));
                timeObj.put("ed", DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss"));
            }
        }
        manData.setTimeDesc(timeObj.toJSONString());//邀访时间格式  需要根据两个字段以及数据字典中对应的类型进行计算得到开始和结束时间
        manData.setVisitorType("3");//访客类型
        //到访事由   两个字段做或判断
        if (StringUtils.isNotBlank(infoMation.getVisitReason())) {
            manData.setContent(infoMation.getVisitReason());
        } else {
            manData.setContent(infoMation.getVisitReasonOther());
        }
        //访问区域设置
        if (StringUtils.isNotBlank(infoMation.getVisitRegionCode())) {
            String[] codeArray = infoMation.getVisitRegionCode().split(",");
            if (codeArray.length == 1) {
                //判断设备是办公楼 则认为是全部设备，相反则只有门口的设备
                if (StringUtils.equals("0", codeArray[0])) {
                    manData.setDeviceApList(deviceGuidAllArray);
                } else {
                    manData.setDeviceApList(deviceGuidInArray);
                }
            } else if (codeArray.length == 2) {
                //两个都选了，则认为是所有设备全部开启
                manData.setDeviceApList(deviceGuidAllArray);
            }
        } else {
            //默认只给门口设备的权限
            manData.setDeviceApList(deviceGuidInArray);
        }
        List<VisitDataSubItemDto> userList = new ArrayList<>();
        if (infoMation.getUserList() != null && !infoMation.getUserList().isEmpty()) {
            List<VisitUser> userDataList = infoMation.getUserList();
            for (VisitUser user : userDataList) {
                VisitDataSubItemDto userItemDto = new VisitDataSubItemDto();
                //此处的访客ID存在问题，一会沟通下
                userItemDto.setVisitorId(user.getPersonId());//访客ID
                userItemDto.setVisitorName(user.getUserName());//访客名称
                userItemDto.setVisitorTel(user.getVisitPhone());//访客手机号
                //下面需要构建凭证信息，此处没有用数据获取的源头
                List<VisitDataCredentialDto> credentialDtos = new ArrayList<>();
                VisitDataCredentialDto credentialDto = new VisitDataCredentialDto();
                credentialDto.setCredentialId(UUID.randomUUID().toString());
                credentialDto.setCredentialType("1");
                String cardNo = RandomUtil.randomString("ABCDEF0123456789", 8);
                credentialDto.setCredentialNo(cardNo);
                credentialDtos.add(credentialDto);
                userItemDto.setCredentialList(credentialDtos);
                ///参数对应啦
                userList.add(userItemDto);
            }
        }
        result.put("visitDataInviteModel", manData);
        result.put("visitDataSubItemModelList", userList);
        return result;
    }


    /***
     * 拉取访客所需的二维码信息
     * @return
     */
    @Override
    public String pullVisitUserQrCode(String visitUserId) {
        String apiUri = "/api/base/getqrcodedata";
        HttpHeaders headers = this.loadHeaderHandle();
        if (headers != null) {
            JSONObject pushData = new JSONObject();
            pushData.put("guid", visitUserId);
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData, headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    log.info("pullVisitUserQrCode接口返回信息："+responseEntity.getBody().toJSONString());
                    JSONObject body = responseEntity.getBody();
                    if(body.containsKey("code") && StringUtils.equals("0",body.getString("code"))){
                        JSONObject data = body.getJSONObject("data");
                        if(data.containsKey("qrCodeData")){
                            return data.getString("qrCodeData");
                        } else {
                            return null;
                        }
                    }else{
                        log.error("pullVisitUserQrCode接口返回错误信息："+body.getString("msg"));
                        //请求返回结果错误啦
                        return null;
                    }
                } else {
                    //请求失败啦
                    log.error("pullVisitUserQrCode接口请求失败");
                    return null;
                }
            } catch (Exception ex) {
                log.error(ex.getMessage(), ex);
                return null;
            }
        } else {
            //认证信息获取失败
            log.error("pullVisitUserQrCode接口获取认证参数失败");
            return null;
        }
    }


    /**
     * 保存门禁访问记录
     *
     * @param recordDto
     * @return
     */
    @Override
    public JieLinkResultDto saveDoorRecordInfo(JieLinkDoorRecordDto recordDto) {
        /**
         * 开始处理上传的门禁记录
         * 1、检查访客人员是否存在，不存在是否还需要新增
         * 2、检查设备系统内是否存在，不存在返回错误信息
         * 3、保存门禁记录信息，按照年划分
         */
        JieLinkResultDto result = new JieLinkResultDto();
        JieLinkPersonDto person = recordDto.getPerson();
        SysUser sysUserData = null;
        if (person != null) {
            String personId = person.getPersonId();
            if (StringUtils.isNotBlank(personId)) {
                sysUserData = sysUserService.getUserDataByPersonId(personId);
               /* if (sysUserData == null) {
                    // //将用户添加到SYS_USER表
                    // sysUserData = new SysUser();
                    // sysUserData.setPersonId(personId);//对应ID
                    // sysUserData.setName(person.getPersonName());//名称
                    // sysUserData.setUserType("1");//默认定义为游客用户
                    // sysUserData.setEmail(person.getEmail());//邮箱
                    // sysUserData.setPhone(person.getMobile());//手机号
                    // sysUserData.setSex(person.getPersonGender() == 0 ? "1" : "0");//性别
                    // //证件信息
                    // if (StringUtils.equals(person.getCertificateType(), "GENERIDENT") || StringUtils.equals(person.getCertificateType(), "IDENTITY")) {
                    //     sysUserData.setIdNumber(person.getIdentityNo());
                    // } else {
                    //     log.error("添加的用户使用的是非身份证信息，暂时将其写入到身份证字段中");
                    //     sysUserData.setIdNumber(person.getCertificateType() + "_" + person.getIdentityNo());
                    // }
                    // //地址
                    // sysUserData.setAddress(person.getAddress());//
                    // //保存数据到用户表中
                    // sysUserService.persist(sysUserData);
                    // //此处还需要查询一次用户的ID
                    // sysUserData = sysUserService.getUserDataByPersonId(personId);
                }*/
            } else {
                //访客ID无了
                result.setCode("ILLEGAL_ARGUMENT");
                result.setMsg("人员信息丢失，请检查参数");
                return result;
            }
        } else {
            //访客记录为空了
            result.setCode("ILLEGAL_ARGUMENT");
            result.setMsg("人员信息丢失，请检查参数");
            return result;
        }
        JieLinkDeviceDto device = recordDto.getDevice();
        Devices devices = null;
        if (device != null) {
            if (StringUtils.isNotBlank(device.getDeviceGuid())) {
                Devices deviceInfo = devicesService.getDeviceInfoByDeviceGuid(device.getDeviceGuid());
                if (deviceInfo != null) {
                    devices = deviceInfo;
                } else {
                    //设备无了
                    result.setCode("ILLEGAL_ARGUMENT");
                    result.setMsg("设备未获取或关键信息错误，请检查参数");
                    return result;
                }
            } else {
                //设备无了
                result.setCode("ILLEGAL_ARGUMENT");
                result.setMsg("设备关键信息丢失，请检查参数");
                return result;
            }
        } else {
            //设备无了
            result.setCode("ILLEGAL_ARGUMENT");
            result.setMsg("设备信息丢失，请检查参数");
            return result;
        }
        DoorRecord record = new DoorRecord();
        record.setRecordId(recordDto.getRecordId());
        if(sysUserData!=null){
            record.setUserId(sysUserData.getBusinessId());
            record.setPersonId(sysUserData.getPersonId());
            record.setMobile(sysUserData.getPhone());
            record.setTenementType(sysUserData.getUserType());
        }else{
            record.setPersonId(person.getPersonId());
            record.setMobile(person.getMobile());
            if(person.getTenementType()==5){
                record.setTenementType("1");
            }else{
                record.setTenementType("0");
            }

        }
        record.setRemark(recordDto.getRemark());
        record.setDeviceId(device.getDeviceGuid());//需要根据设备补全
        record.setDeviceName(device.getDeviceName());//需要根据设备补全
        //TODO 此处字段需要后续修改，需要根据业务调整
        record.setDeviceType(devices.getDeviceType() + "");//需要根据设备不全
        record.setCardType(recordDto.getCardType() + "");
        record.setCardNo(recordDto.getCardNo());
        if (StringUtils.isNotBlank(recordDto.getCrossTime())) {
            DateTime crossTime = DateUtil.parse(recordDto.getCrossTime(), "yyyy-MM-dd HH:mm:ss");
            record.setCrossTime(crossTime.toJdkDate());
        } else {
            //触发时间无了
            result.setCode("ILLEGAL_ARGUMENT");
            result.setMsg("闸机触发时间丢失，请检查参数");
            return result;

        }
        record.setRecordType(recordDto.getRecordType() + "");
        record.setEventType(recordDto.getEventType() + "");
        record.setFlag("1");
        record.setDelFlag("0");
        record.setCreateBy(null);
        record.setCreateDate(new Date());
        record.setUpdateDate(record.getCreateDate());
        doorRecordService.saveRecordData(record);
        result.setCode("0");
        result.setMsg("");
        return result;
    }

    @Override
    public void syncDeviceInfo() {
        String apiUri = "/api/base/devices";
        HttpHeaders headers = this.loadHeaderHandle();
        if (headers != null) {
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(null, headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try {
                if (HttpStatus.OK.value() == responseEntity.getStatusCode().value()) {
                    log.info("syncDeviceInfo接口返回信息：" + responseEntity.getBody().toJSONString());
                    JSONObject body = responseEntity.getBody();
                    if (StringUtils.equals(body.getString("code"), "0")) {
                        JSONObject data = body.getJSONObject("data");
                        if (data.containsKey("devices") && StringUtils.isNotBlank(data.getString("devices"))) {
                            List<Devices> devicesList = JSONArray.parseArray(data.getJSONArray("devices").toJSONString(), Devices.class);
                            if (devicesList != null && !devicesList.isEmpty()) {
                                List<Devices> searchList = devicesService.findAll(null);
                                Map<String, Devices> searchMap = new HashMap<>();
                                for (Devices d : searchList) {
                                    searchMap.put(d.getDeviceGuid(), d);
                                }
                                for (Devices inDevice : devicesList) {
                                    inDevice.setDeviceType(null);
                                    inDevice.setDeviceioType(null);
                                    if (searchMap.containsKey(inDevice.getDeviceGuid())) {
                                        devicesService.merge(inDevice);
                                    } else {
                                        devicesService.persist(inDevice);
                                    }
                                }
                            }
                        }
                    } else {
                        log.error("syncDeviceInfo接口返回错误信息：" + body.getString("msg"));
                    }
                } else {
                    log.error("syncDeviceInfo接口请求失败");
                }
            } catch (Exception ex) {
                log.error(ex.getMessage(), ex);
            }
        } else {
            //认证信息获取失败
            log.error("syncDeviceInfo接口获取认证参数失败");
        }
    }

    @Override
    public void getDeptList() {
        String apiUri = "/api/base/depts";
        HttpHeaders headers = this.loadHeaderHandle();
        if(headers!=null){
            JSONObject pushData = new JSONObject();
            pushData.put("pageIndex",1);
            pushData.put("pageSize",300);
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData,headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    log.info("getDeptList接口返回信息："+responseEntity.getBody().toJSONString());
                    JSONObject body = responseEntity.getBody();
                    if(StringUtils.equals(body.getString("code"),"0")){

                    }else{
                        log.error("syncDeviceInfo接口返回错误信息："+body.getString("msg"));
                    }
                }else{
                    log.error("syncDeviceInfo接口请求失败");
                }
            }catch (Exception ex){
                log.error(ex.getMessage(),ex);
            }
        }else{
            //认证信息获取失败
            log.error("syncDeviceInfo接口获取认证参数失败");
        }
    }

    @Override
    public String loadAllUserInfo(int pageIndex) {
        String apiUri = "/api/base/personlist";
        String resultStr = "";
        HttpHeaders headers = this.loadHeaderHandle();
        if(headers!=null){
            JSONObject pushData = new JSONObject();
            pushData.put("pageIndex",pageIndex);
            pushData.put("pageSize",5);
            HttpEntity<JSONObject> httpEntity = new HttpEntity<>(pushData,headers);
            StringBuffer uriAddr = new StringBuffer();
            uriAddr.append(jieLinkUri).append(apiUri);
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(uriAddr.toString(), httpEntity, JSONObject.class);
            try{
                if(HttpStatus.OK.value() ==responseEntity.getStatusCode().value()) {
                    log.info("getDeptList接口返回信息："+responseEntity.getBody().toJSONString());
                    JSONObject body = responseEntity.getBody();
                    if (StringUtils.equals(body.getString("code"), "0")) {
                        Thread thread=new Thread(new Runnable() {
                            @Override
                            public void run() {
                                JSONArray list = body.getJSONObject("data").getJSONArray("personList");
                                for (int i = 0; i < list.size(); i++) {
                                    //3、把里面的对象转化为JSONObject
                                    JSONObject job = list.getJSONObject(i);
                                    SysUser u = new SysUser();
                                    u.setPersonId(job.getString("personId"));
                                    u.setUserType("0");
                                    u.setUsername(job.getString("personNo"));
                                    u.setName(job.getString("personName"));
                                    u.setPhone(job.getString("mobile"));
                                    u.setSex(job.getString("personGender"));
                                    u.setNltBz("112");
                                    u.setNltBzLabel("上午8:30，下午16:30");
                                    sysUserRepositorys.addSysUsers(u);
                                }
                            }
                        });
                        thread.start();
                        resultStr = body.getString("data");
                    }else{
                        log.error("syncDeviceInfo接口返回错误信息："+body.getString("msg"));
                    }
                }else{
                    log.error("syncDeviceInfo接口请求失败");
                }
            }catch (Exception ex){
                log.error(ex.getMessage(),ex);
            }
        }else{
            //认证信息获取失败
            log.error("syncDeviceInfo接口获取认证参数失败");
        }
        return resultStr;
    }


    /**
     * 创建hander头信息
     *
     * @return
     */
    private HttpHeaders loadHeaderHandle() {
        //加载头部信息
        if (SignUtils.signData != null) {
            SignDto signData = SignUtils.getSignData(SignUtils.signData);
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
            headers.set("Accept", MediaType.APPLICATION_JSON_VALUE);
            headers.set("appId", signData.getAppId());
            headers.set("v", signData.getV());
            headers.set("random", signData.getRandom());
            headers.set("timestamp", signData.getTimestamp());
            headers.set("sign", signData.getSign());
            return headers;
        } else {
            SignDto appKeyInfo = this.getAppKeyInfo();
            if (appKeyInfo != null) {
                SignUtils.signData = appKeyInfo;
                SignDto signData = SignUtils.getSignData(appKeyInfo);
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
                headers.set("Accept", MediaType.APPLICATION_JSON_VALUE);
                headers.set("appId", signData.getAppId());
                headers.set("v", signData.getV());
                headers.set("random", signData.getRandom());
                headers.set("timestamp", signData.getTimestamp());
                headers.set("sign", signData.getSign());
                return headers;
            } else {
                return null;
            }
        }
    }

    /**
     * 将文件转换为base64字符串
     *
     * @param filePath
     * @return
     */
    public String encryptToBase64(String filePath) {
        if (filePath == null) {
            return null;
        }
        try {
            byte[] b = Files.readAllBytes(Paths.get(filePath));
            return Base64.getEncoder().encodeToString(b);
        } catch (IOException e) {
            log.error("读取用户头像文件失败：" + e.getMessage(), e);
        }
        return null;
    }


}
