package org.rcisoft.integration.video.config;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.sun.jna.Pointer;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.integration.video.lib.NetSDKLib;
import org.rcisoft.integration.video.utils.SpringBeanUtils;
import org.rcisoft.sys.doorreportdata.entity.DoorReportData;
import org.rcisoft.sys.doorreportdata.service.DoorReportDataService;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.Date;

/**
 * 抓拍数据回调方法
 */
@Slf4j
public class CapturePicCallBack implements NetSDKLib.fSnapRev{

    /**
     * 服务器内存储的根路径
     */
    private String basePath;

    /**
     * 文件存储的路径
     */
    private String savePath;

    /**
     * 需要操作的数据
     */
    private DoorReportData reportData;


    private DoorReportDataService doorReportDataService = SpringBeanUtils.getBean(DoorReportDataService.class);


    public CapturePicCallBack(String basePath, String savePath, DoorReportData reportData) {
        this.basePath = basePath;
        this.savePath = savePath;
        this.reportData = reportData;
    }

    /**
     * 抓图回调方法
     * @param lLoginID
     * @param pBuf
     * @param RevLen
     * @param EncodeType
     * @param CmdSerial
     * @param dwUser
     */
    @Override
    public void invoke(NetSDKLib.LLong lLoginID, Pointer pBuf, int RevLen, int EncodeType, int CmdSerial, Pointer dwUser) {
        BufferedImage bufferedImage = null;
        if(pBuf != null && RevLen > 0) {
            String savePicPath =basePath+File.separator+ savePath;
            String picFileName = IdUtil.simpleUUID()+".JPG";
            if(!FileUtil.exist(savePicPath)){
                FileUtil.mkdir(savePicPath);
            }
            String strFileName = savePicPath+File.separator+ picFileName;
            byte[] buf = pBuf.getByteArray(0, RevLen);
            ByteArrayInputStream byteArrInput = new ByteArrayInputStream(buf);
            try {
                bufferedImage = ImageIO.read(byteArrInput);
                if(bufferedImage == null) {
                    return;
                }
                ImageIO.write(bufferedImage, "jpg", new File(strFileName));
                //开始保存数据到表中
                String picRelativePath = savePath+File.separator+picFileName;
                picRelativePath = picRelativePath.replaceAll("\\\\","/");
                //转换文件的路径，从存储路径转换到访问路径
                reportData.setCapturePic(picRelativePath);
                reportData.setCaptureTime(new Date());
                doorReportDataService.merge(reportData);
            } catch (IOException e) {
                log.error("抓图回调函数出现异常："+e.getMessage(),e);
            }
        }
    }

    public String getSavePath() {
        return savePath;
    }

    public void setSavePath(String savePath) {
        this.savePath = savePath;
    }

    public DoorReportData getReportData() {
        return reportData;
    }

    public void setReportData(DoorReportData reportData) {
        this.reportData = reportData;
    }
}
